/**
 * Copyright Copyright 2007-8 Simon Andrews
 *
 *    This file is part of StackMeasure.
 *
 *    StackMeasure is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    StackMeasure is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;


public class PreferencesPanel extends JPanel implements KeyListener {

	private StackMeasurePreferences prefs;
	private JTextField hueFlexibilityArea;
	private JTextField minIntensityArea;
	private JTextField minPixelsArea;
	private JTextField pixelDistanceArea;
	private JTextField stackDistanceArea;
	private Vector listeners = new Vector();

	
	public PreferencesPanel (StackMeasurePreferences prefs) {
		this.prefs = prefs;
		
		setLayout(new GridBagLayout());
		
		setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY));
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx=1;
		gbc.gridy=1;
		gbc.weightx=0.5;
		gbc.weighty=0.01;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(5,5,5,5);

		gbc.gridwidth = 1;
		add(new JLabel("Preferences",JLabel.CENTER),gbc);
		gbc.gridy++;
		gbc.weighty=0.5;
		
		add(new JLabel("Min Intensity (%)"),gbc);
		gbc.gridx++;
		minIntensityArea = new JTextField(""+prefs.minIntensity());
		minIntensityArea.addKeyListener(this);
		add(minIntensityArea,gbc);
		
		gbc.gridx=1;
		gbc.gridy++;

		add(new JLabel("Hue Flexibility (%)"),gbc);
		gbc.gridx++;
		hueFlexibilityArea = new JTextField(""+prefs.hueFlexibility());
		hueFlexibilityArea.addKeyListener(this);
		add(hueFlexibilityArea,gbc);

		gbc.gridx=1;
		gbc.gridy++;

		add(new JLabel("Min pixels per group"),gbc);
		gbc.gridx++;
		minPixelsArea = new JTextField(""+prefs.minPixels());
		minPixelsArea.addKeyListener(this);
		add(minPixelsArea,gbc);

		gbc.gridx=1;
		gbc.gridy++;

		add(new JLabel("Distance between pixels ("+"\u00B5"+"m)"),gbc);
		gbc.gridx++;
		pixelDistanceArea = new JTextField(""+prefs.pixelDistance());
		pixelDistanceArea.addKeyListener(this);
		add(pixelDistanceArea,gbc);

		gbc.gridx=1;
		gbc.gridy++;

		add(new JLabel("Distance between stacks ("+"\u00B5"+"m)"),gbc);
		gbc.gridx++;
		stackDistanceArea = new JTextField(""+prefs.stackDistance());
		stackDistanceArea.addKeyListener(this);
		add(stackDistanceArea,gbc);

		

	}
	
	public void addListener (PreferencesListener l) {
		if (!listeners.contains(l)) {
			listeners.add(l);
		}
	}
	public void removeListener (PreferencesListener l) {
		if (listeners.contains(l)) {
			listeners.remove(l);
		}
	}
	
	private void preferencesOK () {
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((PreferencesListener)(e.nextElement())).preferencesOK();
		}
	}
	private void preferencesNotOK () {
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((PreferencesListener)(e.nextElement())).preferencesNotOK();
		}
	}

	public void checkReady () {
		// We just need to know if everything's OK in the values we're asking for.
		
		// Check there's something in each of them first
		if (minIntensityArea.getText().length()==0 || hueFlexibilityArea.getText().length()==0) {
			preferencesNotOK();
			return;
		}
		
		// Now try to actually read the data
		try {
			prefs.setMinIntensity(Integer.parseInt(minIntensityArea.getText()));
			prefs.setHueFlexibility(Integer.parseInt(hueFlexibilityArea.getText()));
			prefs.setMinPixels(Integer.parseInt(minPixelsArea.getText()));
			prefs.setPixelDistance(Double.parseDouble(pixelDistanceArea.getText()));
			prefs.setStackDistance(Double.parseDouble(stackDistanceArea.getText()));
		}
		catch (NumberFormatException e) {
			preferencesNotOK();
			return;
		}
		catch (IllegalArgumentException e) {
			preferencesNotOK();
			return;
		}
				
		// OK everything's OK
		preferencesOK();
	}


	public void keyPressed(KeyEvent arg0) {

	}


	public void keyReleased(KeyEvent arg0) {
		checkReady();	
	}


	public void keyTyped(KeyEvent arg0) {
	}
	
}
