/**
 * Copyright Copyright 2007-8 Simon Andrews
 *
 *    This file is part of StackMeasure.
 *
 *    StackMeasure is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    StackMeasure is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.UIManager;

import ij.ImagePlus;
import ij.plugin.filter.PlugInFilter;
import ij.process.ImageProcessor;


public class StackMeasure_ extends JDialog implements PlugInFilter, ActionListener, PreferencesListener, FilesListener,StackMeasureProcessorListener {

	private GroupCollection collection;
	private JButton runSearchButton;
	private StackMeasurePreferences prefs = new StackMeasurePreferences();
	private FileList fileList = new FileList();
	private boolean prefsOK = true;
	private boolean filesOK = false;
	
	private static boolean launchedDirectly = false;
	
	public StackMeasure_ (){
		setTitle("Stack Measurer");
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {}
				
		getContentPane().setLayout(new BorderLayout());
		
		PreferencesPanel optionsPanel = new PreferencesPanel(prefs);
		optionsPanel.addListener(this);
		getContentPane().add(optionsPanel,BorderLayout.EAST);
		
		FileListPanel filePanel = new FileListPanel(fileList);
		filePanel.addListener(this);
		getContentPane().add(filePanel,BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel();
		JButton closeButton = new JButton("Close");
		closeButton.setActionCommand("close");
		closeButton.addActionListener(this);
		buttonPanel.add(closeButton);
		
		runSearchButton = new JButton("Start Search");
		runSearchButton.setActionCommand("search");
		runSearchButton.addActionListener(this);
		runSearchButton.setEnabled(false);
		buttonPanel.add(runSearchButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setSize(600,300);
		setLocationRelativeTo(null);
		if (launchedDirectly) {
			setDefaultCloseOperation(JDialog.EXIT_ON_CLOSE);
		}
		else {
			setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		}
		
		setVisible(true);
		
	}
	
	public void run(ImageProcessor arg0) {
		// We're not really a filter, so we don't do anything here.
	}

	public int setup(String options, ImagePlus imagePlus) {
				
		return DONE;
	}
	
	private void checkReady () {
		if (prefsOK && filesOK) {
			runSearchButton.setEnabled(true);
		}
		else {
			runSearchButton.setEnabled(false);
		}
	
	}

	public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("close")) {
			setVisible(false);
			dispose();
		}
		else if (ae.getActionCommand().equals("search")) {
			runSearchButton.setEnabled(false);
			collection = new GroupCollection(prefs,fileList);
			StackMeasureProcessor processor = new StackMeasureProcessor(collection);
			processor.addSparkMeasureProcessorListener(this);
			processor.addSparkMeasureProcessorListener(new ProgressDialog(this));
			processor.startProcessing(fileList);
		}
		else {
			System.err.println("Unknown action command '"+ae.getActionCommand()+"'");
		}
	}

	
	public static void main (String [] args) {
		launchedDirectly = true;
		new StackMeasure_();
	}

	public void exceptionReceived(Exception e) {
		e.printStackTrace();
		runSearchButton.setEnabled(true);
	}

	public void processingComplete() {
		new CombinedResultsWindow(collection);
		setVisible(false);
		dispose();
	}

	public void progressUpdate(int current, int total, String message) {
	}

	public void preferencesNotOK() {
		prefsOK = false;
		checkReady();
	}

	public void preferencesOK() {
		prefsOK = true;
		checkReady();
	}

	public void filesNotOK() {
		filesOK = false;
		checkReady();
	}

	public void filesOK() {
		filesOK = true;
		checkReady();
	}

}
